<?php

namespace DMG\Wires;

/**
 * Class Cron
 *
 * Register all cron jobs
 *
 * @package DMG\Wires
 */
class Cron {

	/**
	 * Cron name for ingestion
	 * 
	 * @var string
	 */
	const NAME_INGESTION = 'dmg_wires_ingestion';

	/**
	 * Cron name for cleanup
	 * 
	 * @var string
	 */
	const NAME_CLEANUP = 'dmg_wires_cleanup';

	/**
	 * Schedule name for every minute
	 * 
	 * @var string
	 */
	const SCHEDULE_EVERY_MINUTE = 'dmg_wires_ingest_every_minute';

	/**
	 * Init
	 */
	public static function init() {
		add_filter( 'cron_schedules', array( __CLASS__, 'filter_schedules' ) );
		add_filter( 'wpcom_vip_passthrough_cron_to_jobs', array( __CLASS__, 'passthrough_to_jobs' ) );
		add_action( 'admin_init', array( __CLASS__, 'schedule' ) );
		add_action( self::NAME_INGESTION, array( __CLASS__, 'ingest' ) );
		add_action( self::NAME_CLEANUP, array( __CLASS__, 'cleanup' ) );
	}

	/**
	 * Add a cron schedule for every minute
	 *
	 * @param array $schedules The existing cron schedules
	 * @return array The filtered cron schedules
	 */
	public static function filter_schedules( $schedules ) {
		$schedules[ self::SCHEDULE_EVERY_MINUTE ] = [
			'interval' => MINUTE_IN_SECONDS,
			'display'  => 'Every Minute',
		];

		return $schedules;
	}

	/**
	 * Pass crons off to WP.com jobs system
	 *
	 * @param array $whitelist Crons running on jobs system
	 * @return array Filtered crons on jobs system
	 */
	public function passthrough_to_jobs( $whitelist ) {
		$whitelist[] = self::NAME_INGESTION;
		$whitelist[] = self::NAME_CLEANUP;

		return $whitelist;
	}

	/**
	 * Schedule the cron jobs
	 */
	public static function schedule() {

		// Schedule ingestion to run every miute
		if ( ! wp_next_scheduled( self::NAME_INGESTION ) ) {
			wp_schedule_event( time(), self::SCHEDULE_EVERY_MINUTE, self::NAME_INGESTION );
		}

		// Schedule cleanup to run every day
		if ( ! wp_next_scheduled( self::NAME_CLEANUP ) ) {
			wp_schedule_event( time(), 'daily', self::NAME_CLEANUP );
		}
	}

	/**
	 * Ingestion
	 */
	public static function ingest() {

		// Get registered services
		$services = Services::get_instance()::get_services();
		
		// Run ingestion method for each service
		foreach ( $services as $service ) {
			if ( method_exists( $service, 'ingest_posts' ) ) {
				$service->ingest_posts();
			}
		}
	}

	/**
	 * Cleanup
	 */
	public static function cleanup() {

		// Query posts older than 7 days
		$query = new \WP_Query(
			array(
				'post_type'      => Post_Type::NAME,
				'post_status'    => 'pending',
				'posts_per_page' => 100,
				'date_query'     => array(
					array(
						'before' => '7 days ago'
					),
				),
			)
		);

		// Permanently delete posts
		if ( $query->have_posts() ) {
			foreach ( $query->posts as $post ) {
				wp_delete_post( $post->ID, true );
			}
		}
	}

}

Cron::init();
