<?php

namespace DMG\Wires\Services;

/**
 * Class PA_Media
 *
 * Register the PA Media service
 *
 * @package DMG\Wires\Services
 */
class PA_Media extends Service {

	/**
	 * The name of the API key option
	 * 
	 * @var string
	 */
	const OPTION_NAME_API_KEY = 'dmg_wires_pa_media_api_key';

	/**
	 * The name of the settings section
	 * 
	 * @var string
	 */
	const SETTINGS_SECTION = 'dmg_wires_pa_media';

	/**
	 * API endpoint
	 * 
	 * @var string
	 */
	const API_ENDPOINT = 'https://content.api.pressassociation.io/v1/item';

	/**
	 * Source term name
	 * 
	 * @var string
	 */
	const SOURCE_NAME = 'PA Media';

	/**
	 * Constructor
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'register_fields' ) );
	}

	/**
	 * Register
	 */
	public function register_fields() {

		// Register a new settings section
		add_settings_section(
			self::SETTINGS_SECTION,
			'PA Media',
			array( $this, 'settings_section_callback' ),
			$this->get_settings_page_name()
		);

		// Register the settings and the corresponding option
		register_setting(
			$this->get_option_group(),
			self::OPTION_NAME_API_KEY,
		);

		// Register API Key field
		add_settings_field(
			self::OPTION_NAME_API_KEY,
			'API Key',
			array( $this, 'api_key_field_callback' ),
			$this->get_settings_page_name(),
			self::SETTINGS_SECTION
		);
	}

	/**
	 * Settings section callback
	 */
	public function settings_section_callback( $args ) {
		echo '<p>Configure settings for the PA Media wire service.</p>';
	}

	/**
	 * API Key field callback function
	 *
	 * @param array $args The field args
	 */
	public function api_key_field_callback( $args ) {
		printf(
			'<input type="password" class="widefat" id="%s" name="%s" value="%s" />',
			esc_attr( self::OPTION_NAME_API_KEY ),
			esc_attr( self::OPTION_NAME_API_KEY ),
			get_option( self::OPTION_NAME_API_KEY )
		);
	}

	/**
	 * Ingest the latest posts
	 */
	public function ingest_posts() {

		// Bail early if no API key set
		$api_key = get_option( self::OPTION_NAME_API_KEY );
		if ( empty( $api_key ) ) {
			return;
		}

		/**
		 * Offer filter for API call query args
		 * 
		 * By default, only query items where type=text
		 * 
		 * @param array $query_args The query args for the API request
		 */
		$query_args = apply_filters(
			'dmg_wires_pa_media_api_query_args',
			array(
				'apikey' => $api_key,
				'type'   => 'text'
			)
		);
		
		$url = add_query_arg( $query_args, self::API_ENDPOINT );

		// Don't cache; we will need up-to-date results
		$response = wp_remote_get( $url );
		if ( is_wp_error( $response ) ) {
			return;
		}

		// JSON-decode body
		$body = wp_remote_retrieve_body( $response );
		$json = json_decode( $body );
		if ( ! is_object( $json ) ) {
			return;
		}

		if ( empty( $json->item )
			|| ! is_array( $json->item )
		) {
			return;
		}

		foreach ( $json->item as $item ) {

			// Check the item URI to see if we've already ingested
			if ( empty( $item->uri )
				|| $this->post_exists( $item->uri )
			) {
				continue;
			}

			$excerpt = '';
			$pieces  = explode( "\n\n", $item->body_text );
			if ( ! empty( $pieces[0] ) ) {
				$excerpt = $pieces[0];
			}

			// Create new post object
			$post_id = wp_insert_post(
				array(
					'post_type'    => \DMG\Wires\Post_Type::NAME,
					'post_title'   => ( ! empty( $item->headline ) ) ? $item->headline : "[UNTITLED] PA Media ${item->uri}",
					'post_content' => $this->get_block_string_from_text( $item->body_text ),
					'post_excerpt' => $excerpt,
					'post_date'    => ( ! empty( $item->versioncreated ) ) ? $item->versioncreated : null,
					'post_status'  => 'pending',
				)
			);

			// If successful, set some additional info
			if ( ! is_wp_error( $post_id ) ) {

				// Set the source ID
				$this->set_source_id( $post_id, $item->uri );

				// Set source term
				$this->set_source_term( $post_id, self::SOURCE_NAME );

				// Set the word count
				$this->set_word_count( $post_id, str_word_count( $item->body_text ) );

				// Loop through all subjects and map them to existing tags/categories
				if ( ! empty( $item->subject )
					&& is_array( $item->subject )
				) {
					$tag_ids      = array();
					$category_ids = array();

					foreach ( $item->subject as $subject ) {

						// Skip a subject that doesn't have a name
						if ( empty( $subject->name ) ) {
							continue;
						}

						$tag = get_term_by( 'name', $subject->name, 'post_tag' );
						if ( is_a( $tag, 'WP_Term' ) ) {
							$tag_ids[] = $tag->term_id;
						}

						$category = get_term_by( 'name', $subject->name, 'category' );
						if ( is_a( $category, 'WP_Term' ) ) {
							$category_ids[] = $category->term_id;
						}
					}

					if ( ! empty( $tag_ids ) ) {
						wp_set_object_terms( $post_id, $tag_ids, 'post_tag', true );
					}

					if ( ! empty( $category_ids ) ) {
						wp_set_object_terms( $post_id, $category_ids, 'category', true );
					}
				}

				/**
				 * Perform an action after the wire has been created
				 * 
				 * @param int $post_id The ID of the newly created wire post
				 */
				do_action( 'dmg_wires_after_create_post', $post_id );
			}
		}
	}
}
