<?php

namespace DMG\Wires\Services;

/**
 * Class Service
 *
 * Abstract class for registering services
 *
 * @package DMG\Wires\Services
 */
abstract class Service {

	/**
	 * Ingest latest posts
	 */
	protected abstract function ingest_posts();

	/**
	 * Check if the item was already ingested
	 * 
	 * @param string $source_id
	 */
	protected function post_exists( $source_id ) {

		// Meta query for a post with the source ID
		$query = new \WP_Query(
			array(
				'post_type'      => \DMG\Wires\Post_Type::NAME,
				'posts_per_page' => 1,
				'post_status'    => 'any',
				'meta_query'     => array(
					array(
						'key'     => \DMG\Wires\Post_Type::META_KEY_SOURCE_ID,
						'value'   => $source_id,
						'compare' => '='
					)
				)
			)
		);

		return $query->have_posts();
	}

	/**
	 * Set a post's source ID
	 * 
	 * @param int    $post_id   The post ID,
	 * @param string $source_id The source ID
	 */
	public function set_source_id( $post_id, $source_id ) {
		update_post_meta( $post_id, \DMG\Wires\Post_Type::META_KEY_SOURCE_ID, $source_id );
	}

	/**
	 * Set a post's word count
	 * 
	 * @param int $post_id The post ID
	 * @param int $count   The word count
	 */
	public function set_word_count( $post_id, $count ) {
		update_post_meta( $post_id, \DMG\Wires\Post_Type::META_KEY_WORD_COUNT, $count );
	}

	/**
	 * Set a post's source term
	 * 
	 * @param int    $post_id     The post ID,
	 * @param string $source_term The source term
	 */
	public function set_source_term( $post_id, $source_term ) {

		$term_id = false;

		// Determine the term ID
		$term = get_term_by( 'name', $source_term, \DMG\Wires\Taxonomy::NAME );

		if ( is_a( $term, 'WP_Term' ) ) {

			// Term already exists
			$term_id = $term->term_id;
		} else {

			// This term hasn't been created yet
			$term = wp_insert_term( $source_term, \DMG\Wires\Taxonomy::NAME );
			if ( ! empty( $term['term_id'] ) ) {
				$term_id = $term['term_id'];
			}
		}

		// Set the object terms
		if ( ! empty( $term_id ) ) {
			wp_set_object_terms( $post_id, $term_id, \DMG\Wires\Taxonomy::NAME );
		}
	}

	/**
	 * Get the DMG Wires option group
	 * 
	 * @return string The name of the option group
	 */
	public function get_option_group() {
		return \DMG\Wires\Settings::OPTION_GROUP;
	}

	/**
	 * Get the DMG Wires settings page name
	 * 
	 * @return string The name of the option group
	 */
	public function get_settings_page_name() {
		return \DMG\Wires\Settings::PAGE_NAME;
	}

	/**
	 * Get string of blocks from basic text string
	 * 
	 * @param string $text The body text
	 * @return string Text converted into string of blocks
	 */
	public function get_block_string_from_text( $text ) {

		// Get pieces
		$pieces = explode( "\n\n", $text );

		// Build basic paragraph blocks
		foreach ( $pieces as &$p ) {
			$p = "<!-- wp:paragraph -->\n<p class=\"\">{$p}</p>\n<!-- /wp:paragraph -->";
		}

		// Rejoin before returning
		return implode( "\n\n", $pieces );
	}
}
