<?php

namespace DMG\Wires\Services;

/**
 * Class SWNS
 *
 * Register the SWNS service
 *
 * @package DMG\Wires\Services
 */
class SWNS extends Service {

	/**
	 * The name of the auth key option
	 * 
	 * @var string
	 */
	const OPTION_NAME_AUTH_KEY = 'dmg_wires_swns_auth_key';

	/**
	 * Option name for sideloading images
	 * 
	 * @var string
	 */
	const OPTION_NAME_SIDELOAD_IMAGES = 'dmg_wires_swns_sideload_images';

	/**
	 * The name of the settings section
	 * 
	 * @var string
	 */
	const SETTINGS_SECTION = 'dmg_wires_swns';

	/**
	 * API endpoint
	 * 
	 * @var string
	 */
	const API_ENDPOINT = 'https://swns.rocks/feeds/feed/story/1/';

	/**
	 * Source term name
	 * 
	 * @var string
	 */
	const SOURCE_NAME = 'SWNS';

	/**
	 * Constructor
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'register_fields' ) );
	}

	/**
	 * Register
	 */
	public function register_fields() {

		// Register a new settings section
		add_settings_section(
			self::SETTINGS_SECTION,
			'SWNS',
			array( $this, 'settings_section_callback' ),
			$this->get_settings_page_name()
		);

		// Register the settings and the corresponding option
		register_setting(
			$this->get_option_group(),
			self::OPTION_NAME_AUTH_KEY,
		);

		// Register Auth Key field
		add_settings_field(
			self::OPTION_NAME_AUTH_KEY,
			'Auth Key',
			array( $this, 'auth_key_field_callback' ),
			$this->get_settings_page_name(),
			self::SETTINGS_SECTION
		);

		register_setting(
			$this->get_option_group(),
			self::OPTION_NAME_SIDELOAD_IMAGES,
		);

		// Register Sideload Images field
		add_settings_field(
			self::OPTION_NAME_SIDELOAD_IMAGES,
			'Sideload Images',
			array( $this, 'sideload_images_field_callback' ),
			$this->get_settings_page_name(),
			self::SETTINGS_SECTION
		);
	}

	/**
	 * Settings section callback
	 */
	public function settings_section_callback( $args ) {
		echo '<p>Configure settings for the SWNS wire service.</p>';
	}

	/**
	 * Auth Key field callback function
	 *
	 * @param array $args The field args
	 */
	public function auth_key_field_callback( $args ) {
		printf(
			'<input type="password" class="widefat" id="%s" name="%s" value="%s" />',
			esc_attr( self::OPTION_NAME_AUTH_KEY ),
			esc_attr( self::OPTION_NAME_AUTH_KEY ),
			get_option( self::OPTION_NAME_AUTH_KEY )
		);
	}

	/**
	 * Sideload images field callback function
	 *
	 * @param array $args The field args
	 */
	public function sideload_images_field_callback( $args ) {
		printf(
			'<input type="checkbox" class="widefat" id="%s" name="%s" %s value="1" />',
			esc_attr( self::OPTION_NAME_SIDELOAD_IMAGES ),
			esc_attr( self::OPTION_NAME_SIDELOAD_IMAGES ),
			checked( get_option( self::OPTION_NAME_SIDELOAD_IMAGES ), true, false )
		);
	}

	/**
	 * Ingest the latest posts
	 */
	public function ingest_posts() {

		// Bail early if no API key set
		$auth_key = get_option( self::OPTION_NAME_AUTH_KEY );
		if ( empty( $auth_key ) ) {
			return;
		}

		$url = self::API_ENDPOINT . $auth_key;

		// Don't cache; we will need up-to-date results
		$response = wp_remote_get( $url );
		if ( is_wp_error( $response ) ) {
			return;
		}

		// Instantiate DOMDocument to parse XML
		$domdocument = new \DOMDocument();
		$domdocument->loadHTML( wp_remote_retrieve_body( $response ) );

		// Loop through each item
		foreach ( $domdocument->getElementsByTagName( 'item' ) as $item )  {

			// Pull out relevant nodes
			$guid        = $item->getElementsByTagName( 'guid' )->item(0)->nodeValue;
			$title       = $item->getElementsByTagName( 'title' )->item(0)->nodeValue;
			$description = $item->getElementsByTagName( 'description' )->item(0)->nodeValue;
			$pubdate     = $item->getElementsByTagName( 'pubdate' )->item(0)->nodeValue;

			// Pull out the HTML content
			$content_node = $item->getElementsByTagName( 'encoded' )->item(0);
			$content = $content_node->ownerDocument->saveHTML( $content_node );

			// Check the item guid to see if we've already ingested
			if ( empty( $guid )
				|| $this->post_exists( $guid )
			) {
				continue;
			}

			$excerpt = '';
			$pieces  = explode( "\n\n", $description );
			if ( ! empty( $pieces[0] ) ) {
				$excerpt = $pieces[0];
			}

			// Clean up some CDATA tags from the text
			$content = str_replace(
				array(
					'<![CDATA[',
					']]>,
					]]&gt;'
				),
				'',
				$content
			);

			// Strip all tags except images
			$content = trim( strip_tags( $content, '<img>' ) );

			// Create new post object
			$post_id = wp_insert_post(
				array(
					'post_type'    => \DMG\Wires\Post_Type::NAME,
					'post_title'   => ( ! empty( $title ) ) ? $title : "[UNTITLED] SWMS {$guid}",
					'post_content' => $this->get_block_string_from_text( $content ),
					'post_excerpt' => strip_tags( $excerpt ),
					'post_date'    => ( ! empty( $pubdate) ) ? date( 'Y-m-d H:i:s', strtotime( $pubdate ) ) : null,
					'post_status'  => 'pending',
				)
			);

			// If successful, set some additional info
			if ( ! is_wp_error( $post_id ) ) {

				// Set the source ID
				$this->set_source_id( $post_id, $guid );

				// Set source term
				$this->set_source_term( $post_id, self::SOURCE_NAME );

				// Set the word count
				$this->set_word_count( $post_id, str_word_count( strip_tags( $content ) ) );

				/**
				 * Perform an action after the wire has been created
				 * 
				 * @param int $post_id The ID of the newly created wire post
				 */
				do_action( 'dmg_wires_after_create_post', $post_id );
			}
		}
	}

	/**
	 * Get string of blocks from basic text string
	 * 
	 * @param string $text  The body text
	 * @return string Text converted into string of blocks
	 */
	public function get_block_string_from_text( $text ) {

		// Get pieces
		$pieces = explode( "\n\n", $text );

		// Build basic paragraph blocks
		foreach ( $pieces as &$p ) {

			// Option to sideload images
			if ( str_contains( $p, '<img' ) ) {

				if ( get_option( self::OPTION_NAME_SIDELOAD_IMAGES ) ) {
				
					// Match on any image sources
					preg_match_all( '/<img.+src="(.+)"/', $p, $matches );

					if ( ! empty( $matches[1] ) ) {

						foreach ( $matches[1] as $match ) {

							// Attempt to sideload images
							$image_id = media_sideload_image(
								$match,
								null,
								null,
								'id'
							);

							$src = wp_get_attachment_image_src( $image_id, 'full' );

							$metadata = wp_get_attachment_metadata( $image_id );

							// Create an images block
							if ( is_int( $image_id )
								&& ! empty( $src[0] )
							) {
								$caption = ( ! empty( $metadata['image_meta']['credit'] ) ) ? $metadata['image_meta']['credit'] : '';

								// Build out the block attributes
								$json = array(
									'url'             => $src[0],
									'caption'         => ! empty( $caption ) ? $caption : '',
									'id'              => (int) $image_id,
									'sizeSlug'        => 'full',
									'linkDestination' => 'none',
								);

								// Build out the shortcode
								$shortcode = sprintf(
									'[caption align="alignnone" attachment_id="%d" id="attachment_%d" width="%d" ]<img src="%s" class="wp-image-%d" width="%d" height="%d"/> %s[/caption]',
									(int) $image_id,
									(int) $image_id,
									(int) $metadata['width'],
									str_replace( '&', '\u0026', $src[0] ),
									(int) $image_id,
									(int) $metadata['width'],
									(int) $metadata['height'],
									$caption
								);

								// Add the full block to the images array
								$p = sprintf( "<!-- wp:image %s -->\n%s\n<!-- /wp:image -->",
									wp_json_encode( $json, JSON_HEX_AMP ),
									$shortcode
								);

							}
						}
					}
				} else {
					$p = '';
				}

			} else {
				$p = "<!-- wp:paragraph -->\n<p class=\"\">{$p}</p>\n<!-- /wp:paragraph -->";
			}
		}

		// Rejoin before returning
		return implode( "\n\n", array_filter( $pieces ) );
	}
}
